"""Various utilities."""
import inspect
from typing import Dict, List, cast
from unittest.mock import Mock, sentinel

from .clients import AbstractLoggerClient


class TestLoggerClient(AbstractLoggerClient):
    """Logger client which helps to test the logging."""

    def __init__(self):
        self.mock = Mock()

    def _call(self, *args, **kwargs) -> None:
        """Check arguments and mark the call."""
        # Get the name of method which called this method.
        function = inspect.stack()[1].function
        # Check arguments matches the method signature.
        sig = inspect.signature(getattr(AbstractLoggerClient, function))
        sig.bind(self, *args, **kwargs)
        # Mark the call into the mock.
        getattr(self.mock, function)(*args, **kwargs)

    def create_log_entry(self, *args, **kwargs) -> str:
        """Create log entry.

        Args:
            service: Name of the logger service.
            log_entry_type: Name of the log entry type.
            source_ip: IP address of the action source.
            session_id: Identifier of the log session.
            content: Input content.
            properties: Input properties.
            references: Input references.
        """
        self._call(*args, **kwargs)
        return cast(str, sentinel.log_entry_id)

    def close_log_entry(self, *args, **kwargs) -> None:
        """Close log entry.

        Args:
            log_entry_id: Identifier of the log entry to close.
            result: Name of the result.
            session_id: Identifier of the log session.
            content: Output content.
            properties: Output properties.
            references: Output references.
        """
        self._call(*args, **kwargs)

    def create_session(self, *args, **kwargs) -> str:
        """Create session.

        Args:
            user_id: User's identifier.
            username: User's username.
        """
        self._call(*args, **kwargs)
        return cast(str, sentinel.session_id)

    def close_session(self, *args, **kwargs) -> None:
        """Close session.

        Args:
            session_id: Identifier of the session to close.
        """
        self._call(*args, **kwargs)

    def get_log_entry_types(self, *args, **kwargs) -> List[str]:
        """Send gRPC request to load log entry types.

        Args:
            service: Identifier of the requested service.
        """
        self._call(*args, **kwargs)
        return []

    def get_services(self, *args, **kwargs) -> List[str]:
        """Send gRPC request to load all defined services.

        Args:
            No arguments
        """
        self._call(*args, **kwargs)
        return []

    def get_results(self, *args, **kwargs) -> Dict[str, int]:
        """Send gRPC request to load all defined results.

        Args:
            service: Identifier of the requested service.
        """
        self._call(*args, **kwargs)
        return {}

    def get_object_reference_types(self, *args, **kwargs) -> List[str]:
        """Send gRPC request to get defined types of the object reference.

        Args:
            No arguments
        """
        self._call(*args, **kwargs)
        return []

    def register_service(self, *args, **kwargs) -> None:
        """Send gRPC request to register a new service.

        Args:
            service: Identifier of the requested service.
            handle: Internal identifier of the requested service.
        """
        self._call(*args, **kwargs)

    def register_results(self, *args, **kwargs) -> None:
        """Send gRPC request to register a new service results.

        Args:
            service: Identifier of the requested service.
            results: Mapping of result names to codes.
        """
        self._call(*args, **kwargs)

    def register_log_entry_types(self, *args, **kwargs) -> None:
        """Send gRPC request to register a new service log entry types.

        Args:
            service: Identifier of the requested service.
            log_entry_types: Iterable with the log entry types.
        """
        self._call(*args, **kwargs)

    def register_object_reference_types(self, *args, **kwargs) -> None:
        """Send gRPC request to register a new reference types.

        Args:
            reference_types: Iterable with the reference types.
        """
        self._call(*args, **kwargs)
